@@ -89,7 +89,7 @@
 #' the row and/or column dendrograms are displayed. If a logical scalar is
 #' provided, it is repeated to become a logical vector of length two.
 #' @param reorderfun function(d, w) of dendrogram and weights for reordering the
-#' row and column dendrograms. The default uses stats{reorder.dendrogram}
+#' row and column dendrograms. The default uses \link[stats]{reorder.dendrogram}
 #'
 #' @param k_row an integer scalar with the desired number of groups by which to
 #' color the dendrogram's branches in the rows (uses \link[dendextend]{color_branches})
@@ -132,7 +132,7 @@
 #' @param scale_fill_gradient_fun A function that creates a smooth gradient for the heatmap.
 #' The default uses \link[ggplot2]{scale_fill_gradientn} with the values of colors, limits, and
 #' na.value that are supplied by the user. The user can input a customized function, such as
-#' \link{scale_color_gradient}() in order to get other results (although the virids default
+#' \link[ggplot2]{scale_color_gradient}() in order to get other results (although the virids default
 #' is quite recommended)
 #'
 #' @param grid_color control the color of the heatmap grid. Default is NA.
@@ -212,9 +212,11 @@
 #' arguments you want based on the ones in \link[webshot]{webshot}.
 #'
 #' Another example: heatmaply(x, file = c("heatmaply_plot.html", "heatmaply_plot.png"))
-#'
-#' @param width,height The width and height of the image saved. Default is
-#' which uses 800 x 500 pixels.
+#' @param width,height The width and height of the output htmlwidget, or the
+#' output file if exporting to png/pdf/etc. Presumed to be in pixels, but
+#' if a plotly internal function decides it's in other units you may end up
+#' with a huge file! Default is 800x500 when exporting to a file, and 100% width
+#' as a htmlwidget.
 #' @param long_data Data in long format. Replaces x, so both should not be used.
 #'  Colnames must be c("name", "variable", "value"). If you do not have a names
 #'  column you can simply use a sequence of numbers from 1 to the number of "rows"
@@ -264,6 +266,11 @@
 #' @param custom_hovertext Custom hovertext matrix (the same dimensions as the input).
 #' If plot_method is "plotly" then just this text is displayed; if plot_method
 #' if "ggplot" then it is appended to the existing text.
+#'
+#' @param suppress_default_hovertext Logical indicating whether to hide the
+#' default hovertext for plot_method = "ggplot" of row, column, value, and Point
+#' size.
+#'
 #' @param label_format_fun Function to format hovertext (eg,
 #'    \code{function(...) round(..., digits=3)} or
 #'    \code{function(...) format(..., digits=3)}
@@ -278,6 +285,7 @@
 #' length of the colorbar/color key relative to the total plot height.
 #' This argument controls the colorbar_len of the side colour plots.
 #' Only used if plot_method = "plotly".
+#' @param plotly_source See \code{source} argument in \code{\link[plotly]{plot_ly}}
 #'
 #' @export
 #' @examples
@@ -424,12 +432,12 @@
 #'
 #'
 #' # Example of removing labels and thus making the plot faster
-#' heatmaply(iris, showticklabels = c(T, F), margins = c(80, 10))
+#' heatmaply(iris, showticklabels = c(TRUE, FALSE), margins = c(80, 10))
 #'
 #' # this is what allows for a much larger matrix to be printed:
 #' set.seed(2017 - 05 - 18)
 #' large_x <- matrix(rnorm(19), 1000, 100)
-#' heatmaply(large_x, dendrogram = F, showticklabels = F, margins = c(1, 1))
+#' heatmaply(large_x, dendrogram = FALSE, showticklabels = FALSE, margins = c(1, 1))
 #' }
 heatmaply <- function(x, ...) {
   UseMethod("heatmaply")
@@ -479,10 +487,6 @@ heatmaply_cor <- function(x,
 
 
 
-
-
-
-
 #' @export
 #' @rdname heatmaply
 heatmaply.default <- function(x,
@@ -572,9 +576,11 @@ heatmaply.default <- function(x,
                               label_format_fun = function(...) format(..., digits = 4),
                               labRow = NULL, labCol = NULL,
                               custom_hovertext = NULL,
+                              suppress_default_hovertext = FALSE,
                               col = NULL,
                               dend_hoverinfo = TRUE,
-                              side_color_colorbar_len = 0.3) {
+                              side_color_colorbar_len = 0.3,
+                              plotly_source = "A") {
   if (!is.null(long_data)) {
     if (!missing(x)) {
       stop("x and long_data should not be used together")
@@ -592,6 +598,27 @@ heatmaply.default <- function(x,
   # this is to fix the error: "argument * matches multiple formal arguments"
   if (!is.null(col)) colors <- col
 
+
+  # informative errors for mis-specified limits
+  if (!is.null(limits)) {
+    if (!is.numeric(limits)) stop("limits must be numeric")
+    if (length(limits) != 2L) stop("limits must be of length 2 (i.e.: two dimensional)")
+
+    r <- range(as.matrix(x), na.rm = TRUE)
+    limits <- sort(limits)
+
+
+    ## Warn for broken heatmap colors
+    if (limits[1] > r[1]) {
+      limits[1] <- r[1]
+      warning("Lower limit is not <= lowest value in x, min of limits is set to the min of the range (otherwise, colors will be broken!)")
+    }
+    if (limits[2] < r[2]) {
+      limits[2] <- r[2]
+      warning("Upper limit is not >= highest value in x, max of limits is set to the max of the range (otherwise, colors will be broken!)")
+    }
+  }
+
   if (is.null(scale_fill_gradient_fun)) {
     if (node_type == "heatmap") {
       scale_fill_gradient_fun <- scale_fill_gradientn(
@@ -621,8 +648,8 @@ heatmaply.default <- function(x,
   if (is.logical(dendrogram)) {
     # Using if and not ifelse to make sure the output is a "scalar".
     dendrogram <- if (dendrogram) "both" else "none"
-    # if(T) "both" else "none"
-    # if(F) "both" else "none"
+    # if (TRUE) "both" else "none"
+    # if (FALSE) "both" else "none"
   }
   dendrogram <- match.arg(dendrogram)
 
@@ -736,6 +763,7 @@ heatmaply.default <- function(x,
     scale = scale,
     na.rm = na.rm,
     custom_hovertext = custom_hovertext,
+    suppress_default_hovertext = suppress_default_hovertext,
     labRow = labRow,
     labCol = labCol,
     ...
@@ -786,8 +814,12 @@ heatmaply.default <- function(x,
     node_type = node_type,
     point_size_name = point_size_name,
     label_format_fun = label_format_fun,
+    suppress_default_hovertext = suppress_default_hovertext,
     dend_hoverinfo = dend_hoverinfo,
-    side_color_colorbar_len = side_color_colorbar_len
+    side_color_colorbar_len = side_color_colorbar_len,
+    plotly_source = plotly_source,
+    height = height,
+    width = width
   )
 
   # TODO: think more on what should be passed in "..."
@@ -863,8 +895,13 @@ heatmaply.heatmapr <- function(x,
                                point_size_name = "Point size",
                                label_format_fun = function(...) format(..., digits = 4),
                                custom_hovertext = x[["matrix"]][["custom_hovertext"]],
+                               suppress_default_hovertext = FALSE,
                                dend_hoverinfo = TRUE,
-                               side_color_colorbar_len = 0.3) {
+                               side_color_colorbar_len = 0.3,
+                               plotly_source = "A",
+                               height = NULL,
+                               width = NULL) {
+
   node_type <- match.arg(node_type)
   plot_method <- match.arg(plot_method)
   cellnote_textposition <- match.arg(
@@ -886,29 +923,9 @@ heatmaply.heatmapr <- function(x,
   }
 
 
-  # informative errors for mis-specified limits
-  if (!is.null(limits)) {
-    if (!is.numeric(limits)) stop("limits must be numeric")
-    if (length(limits) != 2L) stop("limits must be of length 2 (i.e.: two dimensional)")
-
-    r <- range(as.matrix(x$matrix$data), na.rm = TRUE)
-    limits <- sort(limits)
-
-    ## Warn for broken heatmap colors
-    if (limits[1] > r[1]) {
-      limits[1] <- r[1]
-      warning("Lower limit is not <= lowest value in x, min of limits is set to the min of the range (otherwise, colors will be broken!)")
-    }
-    if (limits[2] < r[2]) {
-      limits[2] <- r[2]
-      warning("Upper limit is not >= highest value in x, max of limits is set to the max of the range (otherwise, colors will be broken!)")
-    }
-  }
   if (!is.null(srtRow)) row_text_angle <- srtRow
   if (!is.null(srtCol)) column_text_angle <- srtCol
 
-
-
   # x is a heatmapr object.
   # heatmapr <- list(rows = rowDend, cols = colDend, matrix = mtx, image = imgUri,
   #                  theme = theme, options = options)
@@ -958,7 +975,8 @@ heatmaply.heatmapr <- function(x,
     } else {
       py <- plotly_dend(cols,
         side = "col",
-        dend_hoverinfo = dend_hoverinfo
+        dend_hoverinfo = dend_hoverinfo,
+        plotly_source = plotly_source
       )
     }
   }
@@ -981,7 +999,8 @@ heatmaply.heatmapr <- function(x,
       px <- plotly_dend(rows,
         flip = row_dend_left,
         side = "row",
-        dend_hoverinfo = dend_hoverinfo
+        dend_hoverinfo = dend_hoverinfo,
+        plotly_source = plotly_source
       )
     }
   }
@@ -1007,6 +1026,7 @@ heatmaply.heatmapr <- function(x,
       point_size_name = point_size_name,
       label_format_fun = label_format_fun,
       custom_hovertext = custom_hovertext,
+      suppress_default_hovertext = suppress_default_hovertext,
       showticklabels = showticklabels
     )
   } else if (plot_method == "plotly") {
@@ -1025,7 +1045,10 @@ heatmaply.heatmapr <- function(x,
       colorbar_len = colorbar_len,
       colorbar_thickness = colorbar_thickness,
       custom_hovertext = custom_hovertext,
-      label_format_fun = label_format_fun
+      label_format_fun = label_format_fun,
+      height = height,
+      width = width,
+      plotly_source = plotly_source
     )
   }
 
@@ -1061,7 +1084,8 @@ heatmaply.heatmapr <- function(x,
         fontsize = fontsize_col,
         is_colors = !is.null(RowSideColors),
         colorbar_len = side_color_colorbar_len,
-        label_name = label_names[[1]]
+        label_name = label_names[[1]],
+        plotly_source = plotly_source
       )
     }
   }
@@ -1094,7 +1118,8 @@ heatmaply.heatmapr <- function(x,
         fontsize = fontsize_row,
         is_colors = !is.null(ColSideColors),
         colorbar_len = side_color_colorbar_len,
-        label_name = label_names[[2]]
+        label_name = label_names[[2]],
+        plotly_source = plotly_source
       )
     }
   }
@@ -1103,11 +1128,11 @@ heatmaply.heatmapr <- function(x,
     return(list(p = p, px = px, py = py, pr = pr, pc = pc))
   } else {
     if (!is.null(pc)) {
-      pc <- ggplotly(pc)
+      pc <- ggplotly(pc, source = plotly_source)
       pc <- layout(pc, showlegend = TRUE)
     }
     if (!is.null(pr)) {
-      pr <- ggplotly(pr)
+      pr <- ggplotly(pr, source = plotly_source)
       pr <- layout(pr, showlegend = TRUE)
     }
   }
@@ -1115,15 +1140,21 @@ heatmaply.heatmapr <- function(x,
   ## plotly:
   # turn p, px, and py to plotly objects if necessary
   if (!is.plotly(p)) {
-    p <- ggplotly(p, dynamicTicks = dynamicTicks, tooltip = "text") %>%
-      layout(showlegend = FALSE)
-    ## Currently broken, see:
-    ##  https://github.com/ropensci/plotly/issues/1701
-    # %>%
-    # colorbar(
-    #   len = colorbar_len,
-    #   thickness = colorbar_thickness
-    # )
+    p <- ggplotly(
+        p,
+        dynamicTicks = dynamicTicks,
+        tooltip = "text",
+        height = height,
+        width = width,
+        source = plotly_source) %>%
+        layout(showlegend = FALSE)
+      ## Currently broken, see:
+      ##  https://github.com/ropensci/plotly/issues/1701
+      # %>%
+      # colorbar(
+      #   len = colorbar_len,
+      #   thickness = colorbar_thickness
+      # )
   }
 
   if (draw_cellnote) {
@@ -1167,14 +1198,16 @@ heatmaply.heatmapr <- function(x,
   if (!is.null(px) && !is.plotly(px)) {
     px <- ggplotly(px,
       tooltip = if (dend_hoverinfo) "y" else "none",
-      dynamicTicks = dynamicTicks
+      dynamicTicks = dynamicTicks,
+      source = plotly_source
     ) %>%
       layout(showlegend = FALSE)
   }
   if (!is.null(py) && !is.plotly(py)) {
     py <- ggplotly(py,
       tooltip = if (dend_hoverinfo) "y" else "none",
-      dynamicTicks = dynamicTicks
+      dynamicTicks = dynamicTicks,
+      source = plotly_source
     ) %>%
       layout(showlegend = FALSE)
   }
@@ -1185,7 +1218,7 @@ heatmaply.heatmapr <- function(x,
     title = main, # layout's title: /r/reference/#layout-title
     xaxis = list( # layout's xaxis is a named list. List of valid keys: /r/reference/#layout-xaxis
       title = xlab # xaxis's title: /r/reference/#layout-xaxis-title
-      # showgrid = T        # xaxis's showgrid: /r/reference/#layout-xaxis-showgrid
+      # showgrid = TRUE        # xaxis's showgrid: /r/reference/#layout-xaxis-showgrid
     ),
     yaxis = list( # layout's yaxis is a named list. List of valid keys: /r/reference/#layout-yaxis
       title = ylab # yaxis's title: /r/reference/#layout-yaxis-title
@@ -1240,9 +1273,6 @@ heatmaply.heatmapr <- function(x,
           ticklen = 0
         ))
     }
-    # ggplotly() %>%
-    # layout(yaxis = list(tickmode='auto'),
-    #        xaxis = list(tickmode='auto'))
   }
 
   heatmap_subplot <- heatmap_subplot_from_ggplotly(
@@ -1258,7 +1288,8 @@ heatmaply.heatmapr <- function(x,
     pr = pr,
     pc = pc,
     plot_method = plot_method,
-    showticklabels = showticklabels
+    showticklabels = showticklabels,
+    empty = plotly_empty(source = plotly_source)
   )
   l <- layout(
     heatmap_subplot,
@@ -1304,7 +1335,8 @@ heatmap_subplot_from_ggplotly <- function(p, px, py, pr, pc,
                                           titleX = TRUE, titleY = TRUE,
                                           widths = NULL, heights = NULL,
                                           plot_method,
-                                          showticklabels = c(TRUE, TRUE)) {
+                                          showticklabels = c(TRUE, TRUE),
+                                          empty = plotly_empty(source = "A")) {
   widths <- widths %||% default_dims(px, pr)
   if (row_dend_left) {
     widths <- rev(widths)
@@ -1313,8 +1345,8 @@ heatmap_subplot_from_ggplotly <- function(p, px, py, pr, pc,
 
 
   # make different plots based on which dendrogram and sidecolors we have
-  row1_list <- list(py, plotly_empty(), plotly_empty())
-  row2_list <- list(pc, plotly_empty(), plotly_empty())
+  row1_list <- list(py, empty, empty)
+  row2_list <- list(pc, empty, empty)
   row3_list <- list(p, pr, px)
 
   if (row_dend_left) {
@@ -1394,7 +1426,7 @@ heatmap_subplot_from_ggplotly <- function(p, px, py, pr, pc,
 
   # s <- subplot(plots,
   #   nrows = nrows,
-  #   widths = if(row_dend_left) rev(widths) else widths,
+  #   widths = if (row_dend_left) rev(widths) else widths,
   #   shareX = TRUE, shareY = TRUE,
   #   titleX = titleX, titleY = titleY,
   #   margin = subplot_margin,
@@ -1414,7 +1446,7 @@ calc_margin <- function(labels, fontsize) {
 #' @title Checks if an object is of class plotly or not.
 #' @export
 #' @description
-#' Helpful for the plot_method in link{heatmaply}.
+#' Helpful for the plot_method in \link{heatmaply}.
 #'
 #' @param x an object to check
 #'
